package com.sachin.wifip2p;

import android.Manifest;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.net.NetworkInfo;
import android.net.wifi.p2p.WifiP2pManager;
import android.widget.Toast;

import androidx.core.app.ActivityCompat;

public class WifiBroadcastReceiver extends BroadcastReceiver {
    private WifiP2pManager _wifiP2pManager;
    private WifiP2pManager.Channel _wifiP2pChannel;
    private Wifip2pPlugin _wifip2pPlugin;
    public boolean isConnected = true;

    WifiBroadcastReceiver(Wifip2pPlugin wifiP2pPlugin, WifiP2pManager wifiP2pManager, WifiP2pManager.Channel wifiP2pChannel) {
        _wifip2pPlugin = wifiP2pPlugin;
        _wifiP2pManager = wifiP2pManager;
        _wifiP2pChannel = wifiP2pChannel;
    }

    @Override
    public void onReceive(Context context, Intent intent) {
        String action = intent.getAction();
        if (WifiP2pManager.WIFI_P2P_STATE_CHANGED_ACTION.equals(action)) {
            int state = intent.getIntExtra(WifiP2pManager.EXTRA_WIFI_STATE, -1);
            if (_wifip2pPlugin.wifiStateChangeEvent._eventSink != null) {
                if (state == WifiP2pManager.WIFI_P2P_STATE_ENABLED) {
                    _wifip2pPlugin.wifiStateChangeEvent._eventSink.success(true);
                } else {
                    _wifip2pPlugin.wifiStateChangeEvent._eventSink.success(false);
                }
            }
        } else if (WifiP2pManager.WIFI_P2P_PEERS_CHANGED_ACTION.equals(action)) {
            if (_wifiP2pManager != null) {
                //check if does app have location access permission if not then not try to access peers list
                if (ActivityCompat.checkSelfPermission(context, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                    Toast.makeText(context, "Location Permission Not Granted", Toast.LENGTH_SHORT).show();
                    return;
                }
                _wifiP2pManager.requestPeers(_wifiP2pChannel, _wifip2pPlugin.peerListListener);
                Toast.makeText(context, "Peers Fetch Successfully", Toast.LENGTH_SHORT).show();
            }
        } else if (WifiP2pManager.WIFI_P2P_CONNECTION_CHANGED_ACTION.equals(action)) {
            NetworkInfo networkInfo = intent.getParcelableExtra(WifiP2pManager.EXTRA_NETWORK_INFO);
            // provide first event to connectionChangeEvent
            if(networkInfo.isConnected()) {
                isConnected = true;
            } else {
                isConnected = false;
            }
            if (_wifiP2pManager != null && _wifip2pPlugin.connectionStateChangeEvent._eventSink != null) {
                if (isConnected) {
                    _wifip2pPlugin.connectionStateChangeEvent._eventSink.success(true);
                    _wifiP2pManager.requestConnectionInfo(_wifiP2pChannel, _wifip2pPlugin.connectionInfoListener);
                } else {
                    _wifip2pPlugin.connectionStateChangeEvent._eventSink.success(false);
                }
            }
        } else if (WifiP2pManager.WIFI_P2P_THIS_DEVICE_CHANGED_ACTION.equals(action)) {
            //do something
        }
    }
}
