package com.sachin.wifip2p;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.location.LocationManager;
import android.net.wifi.WifiManager;
import android.net.wifi.p2p.WifiP2pConfig;
import android.net.wifi.p2p.WifiP2pDevice;
import android.net.wifi.p2p.WifiP2pDeviceList;
import android.net.wifi.p2p.WifiP2pInfo;
import android.net.wifi.p2p.WifiP2pManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;
import android.provider.Settings;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.core.app.ActivityCompat;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.StreamHandler;

import io.flutter.app.FlutterActivityEvents;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * Wifip2pPlugin
 */
public class Wifip2pPlugin extends StreamHandler implements FlutterPlugin, MethodCallHandler, ActivityAware, FlutterActivityEvents {
    //FlutterActivityEvents
    private MethodChannel methodChannel;

    public EventChannelHandler wifiStateChangeEvent;
    private EventChannelHandler peerChangeEvent;
    public EventChannelHandler connectionStateChangeEvent;
    public EventChannelHandler receiveDataEvent;

    // on/off  wifi
    private WifiManager wifiManager;

    // control wifiP2p
    private WifiP2pManager wifiP2pManager;
    private WifiP2pManager.Channel wifiP2pChannel;
    private WifiBroadcastReceiver wifiBroadcastReceiver;
    private IntentFilter intentFilter;
    private Context context;
    private Activity activity;

    List<WifiP2pDevice> peers = new ArrayList<WifiP2pDevice>();
    ArrayList<String> deviceNameArray;
    WifiP2pDevice[] devicesArray;

    SendReceive sendReceive;
    ServerClass serverClass;
    ClientClass clientClass;


    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        methodChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "wifiP2p");
        methodChannel.setMethodCallHandler(this);

        wifiStateChangeEvent = new EventChannelHandler(new EventChannel(flutterPluginBinding.getBinaryMessenger(), "wifiStateChangeEvent"));
        peerChangeEvent = new EventChannelHandler(new EventChannel(flutterPluginBinding.getBinaryMessenger(), "peerListChangeEvent"));
        connectionStateChangeEvent = new EventChannelHandler(new EventChannel(flutterPluginBinding.getBinaryMessenger(), "connectionChangeEvent"));
        receiveDataEvent = new EventChannelHandler(new EventChannel(flutterPluginBinding.getBinaryMessenger(), "receiveData"));

    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        methodChannel.setMethodCallHandler(null);

        wifiStateChangeEvent.removeHandler();
        peerChangeEvent.removeHandler();
        connectionStateChangeEvent.removeHandler();
        receiveDataEvent.removeHandler();
    }


    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        if (call.method.equals("toggleWifi")) {
            result.success(toggleWifi());
        } else if (call.method.equals("discoverDevices")) {
            result.success(discoverDevices());
        } else if (call.method.equals("connectToDevice")) {
            int position = (int) call.arguments;
            result.success(connectToDevice(position));
        } else if (call.method.equals("sendData")) {
            if (sendReceive != null) {
                String sendData = (String) call.arguments;
                sendReceive.write(sendData.getBytes());
            }
        } else if (call.method.equals("updateWifiStateChangeEvent")) { //set add first event to wifiStateChangeEvent
            if (wifiStateChangeEvent._eventSink != null) {
                wifiStateChangeEvent._eventSink.success(wifiManager.isWifiEnabled());
            }
        } else if (call.method.equals("updateConnectionChangeEvent")) { // set add first event to connectionChangeEvent
            if (wifiBroadcastReceiver != null) {
                connectionStateChangeEvent._eventSink.success(wifiBroadcastReceiver.isConnected);
            }
        } else if(call.method.equals("openLocationSetting")){
            Intent intent = new Intent();
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            intent.setAction(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
            context.startActivity(intent);
        } else if(call.method.equals("openWifiSetting")){
            Intent intent = new Intent();
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            intent.setAction(Settings.ACTION_WIFI_SETTINGS);
            context.startActivity(intent);
        }else {
            result.notImplemented();// cause "MissingPluginException"
        }
    }

    boolean toggleWifi() {
        if (wifiManager == null)
            return false;

        if (wifiManager.isWifiEnabled()) {
            wifiManager.setWifiEnabled(false);
            return false;
        } else {
            wifiManager.setWifiEnabled(true);
            return true;
        }
    }

    WifiP2pManager.PeerListListener peerListListener = new WifiP2pManager.PeerListListener() {
        @Override
        public void onPeersAvailable(WifiP2pDeviceList peersList) {
            if (!peersList.getDeviceList().equals(peers)) {
                peers.clear();
                peers.addAll(peersList.getDeviceList());

                deviceNameArray = new ArrayList<String>();
                devicesArray = new WifiP2pDevice[peersList.getDeviceList().size()];

                int index = 0;
                if (peersList.getDeviceList().size() != 0) {
                    for (WifiP2pDevice device : peersList.getDeviceList()) {
                        deviceNameArray.add(device.deviceName);
                        devicesArray[index] = device;
                        index++;
                    }
                    peerChangeEvent._eventSink.success(deviceNameArray);
                }
            }
            if (peers.size() == 0) {
                Toast.makeText(context, "No Device Found", Toast.LENGTH_SHORT).show();
            }
        }
    };

    public Boolean connectToDevice(int position) {
        final WifiP2pDevice device = devicesArray[position];
        WifiP2pConfig config = new WifiP2pConfig();
        config.deviceAddress = device.deviceAddress;
        final Boolean[] isDeviceConnected = new Boolean[1];
        getLocationPermission();
        if (ActivityCompat.checkSelfPermission(context.getApplicationContext(), Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            return false;
        }
        wifiP2pManager.connect(wifiP2pChannel, config, new WifiP2pManager.ActionListener() {
            @Override
            public void onSuccess() {
                isDeviceConnected[0] = true;
            }

            @Override
            public void onFailure(int reason) {
                isDeviceConnected[0] = false;
            }
        });

        return isDeviceConnected[0];
    }

    //assign connection status to current status of the network
    WifiP2pManager.ConnectionInfoListener connectionInfoListener = new WifiP2pManager.ConnectionInfoListener() {
        @Override
        public void onConnectionInfoAvailable(WifiP2pInfo info) {
            final InetAddress groupOwnerAddress = info.groupOwnerAddress;
            if (info.groupFormed && info.isGroupOwner) {
                serverClass = new ServerClass();
                serverClass.start();
            } else if (info.groupFormed) {
                clientClass = new ClientClass(groupOwnerAddress);
                clientClass.start();
            }
        }
    };


    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        context = binding.getActivity().getApplicationContext();
        wifiManager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);

        wifiP2pManager = (WifiP2pManager) context.getSystemService(Context.WIFI_P2P_SERVICE);
        wifiP2pChannel = (WifiP2pManager.Channel) wifiP2pManager.initialize(context, context.getMainLooper(), null);

        wifiBroadcastReceiver = new WifiBroadcastReceiver(this, wifiP2pManager, wifiP2pChannel);
        intentFilter = new IntentFilter();
        intentFilter.addAction(WifiP2pManager.WIFI_P2P_STATE_CHANGED_ACTION);
        intentFilter.addAction(WifiP2pManager.WIFI_P2P_PEERS_CHANGED_ACTION);
        intentFilter.addAction(WifiP2pManager.WIFI_P2P_CONNECTION_CHANGED_ACTION);
        intentFilter.addAction(WifiP2pManager.WIFI_P2P_THIS_DEVICE_CHANGED_ACTION);

        context.registerReceiver(wifiBroadcastReceiver, intentFilter);
        activity = binding.getActivity();

    }

    //permission require for discovery of app
    public void getLocationPermission() {
        if (ActivityCompat.checkSelfPermission(context.getApplicationContext(), Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_DENIED) {
            ActivityCompat.requestPermissions(activity, new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, 1);
        }
    }

    public int discoverDevices() {
        final int FAILURE = 0;
        final int SUCCESS = 1;
        final int WIFI_IS_OFF = 2;

        getLocationPermission();
        // if location permission not granted then stop the start of discover peers process
        if (ActivityCompat.checkSelfPermission(context.getApplicationContext(), Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            return FAILURE;
        }
        final int[] isDiscoveryStarts = new int[1];
        wifiP2pManager.discoverPeers(wifiP2pChannel, new WifiP2pManager.ActionListener() {
            @Override
            public void onSuccess() {
                isDiscoveryStarts[0] = SUCCESS;
            }

            @Override
            public void onFailure(int reason) {
                if (!wifiManager.isWifiEnabled()) {
                    isDiscoveryStarts[0] = WIFI_IS_OFF;
                    return;
                }
                isDiscoveryStarts[0] = FAILURE;
            }
        });
        return isDiscoveryStarts[0];
    }

    public final int RECEIVE_MESSAGE = 1;
    public String receivedMessage = "";
    Handler handler = new Handler(new Handler.Callback() {
        @Override
        public boolean handleMessage(@NonNull Message msg) {
            switch (msg.what) {
                case RECEIVE_MESSAGE:
                    receivedMessage = new String((byte[])msg.obj,0,msg.arg1);
                    receiveDataEvent._eventSink.success(receivedMessage);
                    break;
            }
            return true;
        }
    });


    @Override
    public void onPause() {
        context.unregisterReceiver(wifiBroadcastReceiver);
    }

    @Override
    public void onResume() {
        context.registerReceiver(wifiBroadcastReceiver, intentFilter);
    }

    @Override
    public void onDestroy() {
        context.unregisterReceiver(wifiBroadcastReceiver);
        if (sendReceive != null) {
            sendReceive.dataSendingThread.quit();
        }
    }


    public class ServerClass extends Thread {
        Socket socket;
        ServerSocket serverSocket;


        @Override
        public void run() {
            try {
                serverSocket = new ServerSocket(8888);
                socket = serverSocket.accept();

                //socket is ready for send and receiving data
                sendReceive = new SendReceive(socket); // only initialization of input and output stream
                sendReceive.start(); // start thread so that it will always wait for receiving message
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public class ClientClass extends Thread {
        Socket socket;
        String hostAdd;

        public ClientClass(InetAddress hostAddress) {
            hostAdd = hostAddress.getHostAddress();
            socket = new Socket();
        }

        @Override
        public void run() {
            try {
                socket.connect(new InetSocketAddress(hostAdd, 8888), 500);

                //socket is now ready for sending and receiving data
                sendReceive = new SendReceive(socket);
                sendReceive.start();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private class SendReceive extends Thread {
        private Socket socket;
        private InputStream inputStream;
        private OutputStream outputStream;

        private HandlerThread dataSendingThread;
        public Handler dataSendingHandler;

        public SendReceive(Socket skt) {
            socket = skt;
            try {
                inputStream = socket.getInputStream();
                outputStream = socket.getOutputStream();
            } catch (IOException e) {
                e.printStackTrace();
            }

            dataSendingThread = new HandlerThread("Send Data");
            dataSendingThread.start();
            dataSendingHandler = new Handler(dataSendingThread.getLooper());
        }

        @Override
        public void run() {
            byte[] buffer = new byte[1024];
            int bytes;
            while (socket != null) {
                try {
                    bytes = inputStream.read(buffer);//block code until message arrives.
                    if (bytes > 0) {
                        handler.obtainMessage(RECEIVE_MESSAGE, bytes, bytes, buffer).sendToTarget();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        public void write(byte[] bytes) {
            // posting task of sending data
            dataSendingHandler.post(new SendData(outputStream, bytes));
        }

    }


    static class SendData implements Runnable {
        OutputStream outputStream;
        byte[] dataToSend;

        SendData(OutputStream outputStream, byte[] data) {
            this.outputStream = outputStream;
            this.dataToSend = data;
        }

        @Override
        public void run() {
            try {
                outputStream.write(dataToSend);
                outputStream.flush();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }


    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }

    @Override
    public void onCreate(Bundle savedInstanceState) {

    }

    @Override
    public void onNewIntent(Intent intent) {

    }

    @Override
    public void onStart() {

    }

    @Override
    public void onPostResume() {
    }

    @Override
    public void onStop() {

    }

    @Override
    public boolean onBackPressed() {
        return false;
    }

    @Override
    public void onUserLeaveHint() {

    }

    @Override
    public void onTrimMemory(int level) {

    }

    @Override
    public void onConfigurationChanged(@NonNull Configuration newConfig) {

    }

    @Override
    public void onLowMemory() {

    }

    @Override
    public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
        return false;
    }

    @Override
    public boolean onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        return false;
    }
}
