import 'dart:async';
import 'package:flutter/services.dart';

class WifiP2P {
  //Methods     1)toggleWifiState  2)discoverDevices      3)connectToDevice         4)sendData     5)openLocationSetting   6)openWifiSetting
  //Streams     1)wifiStateEvent   2)peerListChangeEvent  3)connectionChangeEvent   4)receiveData 
  
  //MethodChannel
  static const MethodChannel _methodChannel = const MethodChannel('wifiP2p');

  //EventChannels
  static const EventChannel _wifiStateChangeEventChannel =
      const EventChannel("wifiStateChangeEvent");
  static const EventChannel _peerListChangeEventChannel =
      const EventChannel("peerListChangeEvent");
  static const EventChannel _connectionChangeEventChannel =
      const EventChannel("connectionChangeEvent");
  static const EventChannel _receiveDataEventChannel =
      const EventChannel("receiveData");

  //Stream
  static Stream<bool>? _wifiStateChangeEvent;
  static Stream<List<String>>? _peerListChangeEvent;
  static Stream<bool>? _connectionChangeEvent;
  static Stream<String>? _receiveData;

  // All Methods 
  
  static Future<bool> toggleWifiState() async {
    final bool wifiStatus = await _methodChannel.invokeMethod('toggleWifi');
    return wifiStatus;
  }

  static Future<int> discoverDevices() async {
    //return   =   failure: 0     success: 1     wifi_is_off: 2
    int isDiscoveryStarted =
        (await _methodChannel.invokeMethod("discoverDevices")) as int;
    return isDiscoveryStarted;
  }

  static Future<bool> connectToDevice(int position) async {
    //return   =   success: true  ,  failure: false
    bool isConnectionSuccessful = (await _methodChannel.invokeMethod(
        "connectToDevice", position)) as bool;
    return isConnectionSuccessful;
  }

  static Future<void> sendData(String dataToSend) async {
    await _methodChannel.invokeMethod("sendData", dataToSend);
  }

  static Future<void> openLocationSetting() async {
    await _methodChannel.invokeMethod("openLocationSetting");
  }

  static Future<void> openWifiSetting() async {
    await _methodChannel.invokeMethod("openWifiSetting");
  }

  static Future<void> _updateWifiStateChangeEvent() async {
    await Future.delayed(Duration(seconds: 2));
    await _methodChannel.invokeMethod("updateWifiStateChangeEvent");
  }

  static Future<void> _updateConnectionStateChangeEvent() async {
    await Future.delayed(Duration(seconds: 2));
    await _methodChannel.invokeMethod("updateConnectionChangeEvent");
  }

  // All Stream Getters

  //notify about change in state of wifi
  static Stream<bool>? get wifiStateChangeEvent {
    _wifiStateChangeEvent ??= _wifiStateChangeEventChannel
        .receiveBroadcastStream()
        .map<bool>((i) => i);
    _updateWifiStateChangeEvent();
    return _wifiStateChangeEvent;
  }

  //notify about new list of peers
  static Stream<List<String>>? get peerListChangeEvent {
    // map((i)=> i as List<Object>);
    _peerListChangeEvent ??= _peerListChangeEventChannel
        .receiveBroadcastStream()
        .map((peerList) =>
            (peerList as List<Object?>).map((i) => i as String).toList());
    return _peerListChangeEvent;
  }

  //notify when device connected or disconnection
  static Stream<bool>? get connectionChangeEvent {
    _connectionChangeEvent ??= _connectionChangeEventChannel
        .receiveBroadcastStream()
        .map<bool>((i) => i as bool);
    _updateConnectionStateChangeEvent();
    return _connectionChangeEvent;
  }

  static Stream<String>? get receiveData {
    _receiveData ??= _receiveDataEventChannel
        .receiveBroadcastStream()
        .map<String>((i) => i as String);
    return _receiveData;
  }
}
